# -*- coding: utf-8 -*-

import datetime

from warnings import warn
from bisect import bisect_right


class Pope:
    def __init__(self, start, end, english_name, regnal_name, personal_name):
        self.start = start
        self.end = end
        self.english_name = english_name
        self.regnal_name = regnal_name
        self.personal_name = personal_name

    def in_inauguration(self, dt):
        start, end = self.start, self.end
        if not start:
            start = datetime.date.min
        if not end:
            end = datetime.date.max
        if isinstance(dt, datetime.datetime):
            return datetime.datetime.combine(start, datetime.datetime.min.time()) <= \
                   dt <= datetime.datetime.combine(end, datetime.datetime.max.time())
        elif isinstance(dt, datetime.date):
            return start <= dt <= end
        else:
            raise TypeError("dt must be an instance of datetime.datetime or datetime.date")

    def __lt__(self, other):
        end = self.end
        if self.end is None:
            if self.start is None:
                return True
            end = self.start
        if isinstance(other, datetime.datetime):
            return datetime.datetime.combine(end, datetime.datetime.max.time()) < other
        elif isinstance(other, datetime.date):
            return end < other
        else:
            raise TypeError("other must be an instance of datetime.datetime or datetime.date")

    def __gt__(self, other):
        start = self.start
        if self.start is None:
            if self.end is None:
                return True
            start = self.end
        if isinstance(other, datetime.datetime):
            return other < datetime.datetime.combine(start, datetime.datetime.min.time())
        elif isinstance(other, datetime.date):
            return other < start
        else:
            raise TypeError("other must be an instance of datetime.datetime or datetime.date")

    def __repr__(self):
        start = self.start.strftime("%d/%m/%Y") if self.start else None
        end = self.end.strftime("%d/%m/%Y") if self.end else None
        return "<Pope: {}, {}-{}>".format(self.english_name, start, end)


class Popet:
    popes = [Pope(start=datetime.date(30, 1, 1), end=datetime.date(66, 12, 31), english_name="St. Peter",
                  regnal_name="PETRUS", personal_name="Simon Peter, Simeon Kephas, Simon The Rock"),
             Pope(start=datetime.date(67, 1, 1), end=datetime.date(78, 12, 31), english_name="St. Linus",
                  regnal_name="LINUS", personal_name="Linus"),
             Pope(start=datetime.date(79, 1, 1), end=datetime.date(77, 12, 31), english_name="St. Anacletus (Cletus) ",
                  regnal_name="ANACLETUS", personal_name="Anacletus"),
             Pope(start=datetime.date(88, 1, 1), end=datetime.date(96, 12, 31), english_name="St. Clement I",
                  regnal_name="CLEMENS", personal_name="Clement"),
             Pope(start=datetime.date(97, 1, 1), end=datetime.date(106, 12, 31), english_name="St. Evaristus (Aristus)",
                  regnal_name="EVARISTUS", personal_name="Aristus"),
             Pope(start=datetime.date(107, 1, 1), end=datetime.date(115, 12, 31), english_name="St. Alexander I",
                  regnal_name="ALEXANDER", personal_name="Alexander"),
             Pope(start=datetime.date(116, 1, 1), end=datetime.date(124, 12, 31), english_name="St. Sixtus I",
                  regnal_name="XYSTUS", personal_name=None),
             Pope(start=datetime.date(125, 1, 1), end=datetime.date(136, 12, 31), english_name="St. Telesphorus",
                  regnal_name="TELESPHORUS", personal_name=None),
             Pope(start=datetime.date(137, 1, 1), end=datetime.date(140, 12, 31), english_name="St. Hyginus",
                  regnal_name="HYGINUS", personal_name=None),
             Pope(start=datetime.date(141, 1, 1), end=datetime.date(154, 12, 31), english_name="St. Pius I",
                  regnal_name="PIUS", personal_name=None),
             Pope(start=datetime.date(155, 1, 1), end=datetime.date(165, 12, 31), english_name="St. Anicetus",
                  regnal_name="ANICETUS", personal_name=None),
             Pope(start=datetime.date(166, 1, 1), end=datetime.date(174, 12, 31), english_name="St. Soter ",
                  regnal_name="SOTERIUS", personal_name=None),
             Pope(start=datetime.date(175, 1, 1), end=datetime.date(188, 12, 31), english_name="St. Eleuterus",
                  regnal_name="ELEUTHERIUS", personal_name=None),
             Pope(start=datetime.date(189, 1, 1), end=datetime.date(198, 12, 31), english_name="St. Victor I",
                  regnal_name="VICTOR", personal_name=None),
             Pope(start=datetime.date(199, 1, 1), end=datetime.date(216, 12, 31),
                  english_name="St. Zephyrinus (Zephyrin)", regnal_name="ZEPHYRINUS", personal_name=None),
             Pope(start=datetime.date(217, 1, 1), end=datetime.date(222, 12, 31), english_name="St. Callixtus I",
                  regnal_name="CALLISTUS", personal_name=None),
             Pope(start=datetime.date(223, 1, 1), end=datetime.date(230, 7, 20), english_name="St. Urban I",
                  regnal_name="URBANUS", personal_name=None),
             Pope(start=datetime.date(230, 7, 21), end=datetime.date(235, 9, 28), english_name="St. Pontian",
                  regnal_name="PONTIANUS", personal_name=None),
             Pope(start=datetime.date(235, 11, 21), end=datetime.date(236, 1, 3), english_name="St. Anterus",
                  regnal_name="ANTERUS", personal_name=None),
             Pope(start=datetime.date(236, 1, 10), end=datetime.date(250, 1, 20), english_name="St. Fabian",
                  regnal_name="FABIANUS", personal_name=None),
             Pope(start=datetime.date(251, 3, 6), end=datetime.date(253, 6, 30), english_name="St. Cornelius",
                  regnal_name="CORNELIUS", personal_name=None),
             Pope(start=datetime.date(253, 6, 25), end=datetime.date(254, 3, 5), english_name="St. Lucius I",
                  regnal_name="LUCIUS", personal_name=None),
             Pope(start=datetime.date(254, 5, 12), end=datetime.date(257, 8, 2), english_name="St. Stephen I",
                  regnal_name="STEPHANUS", personal_name=None),
             Pope(start=datetime.date(257, 8, 30), end=datetime.date(258, 8, 6), english_name="St. Sixtus II",
                  regnal_name="XYSTUS Secundus", personal_name="&nsp;"),
             Pope(start=datetime.date(259, 7, 22), end=datetime.date(268, 12, 26), english_name="St. Dionysius",
                  regnal_name="DIONYSIUS", personal_name=None),
             Pope(start=datetime.date(269, 1, 5), end=datetime.date(274, 12, 30), english_name="St. Felix I",
                  regnal_name="FELIX", personal_name=None),
             Pope(start=datetime.date(275, 1, 4), end=datetime.date(283, 12, 7), english_name="St. Eutychian",
                  regnal_name="EUTYCHIANUS", personal_name=None),
             Pope(start=datetime.date(283, 12, 17), end=datetime.date(296, 4, 22), english_name="St. Caius",
                  regnal_name="CAIUS", personal_name=None),
             Pope(start=datetime.date(296, 6, 30), end=datetime.date(304, 4, 1), english_name="St. Marcellinus",
                  regnal_name="MARCELLINUS", personal_name=None),
             Pope(start=datetime.date(308, 1, 1), end=datetime.date(308, 12, 31), english_name="St. Marcellus I",
                  regnal_name="MARCELLUS", personal_name=None),
             Pope(start=datetime.date(309, 1, 1), end=datetime.date(309, 12, 31), english_name="St. Eusebius",
                  regnal_name="EUSEBIUS", personal_name=None),
             Pope(start=datetime.date(311, 7, 2), end=datetime.date(314, 1, 10),
                  english_name="St. Miltiades (Melchiades)", regnal_name="MILTIADES", personal_name=None),
             Pope(start=datetime.date(314, 1, 31), end=datetime.date(335, 12, 31), english_name="St. Sylvester I ",
                  regnal_name="SILVESTER", personal_name=None),
             Pope(start=datetime.date(336, 1, 18), end=datetime.date(336, 10, 7), english_name="St. Mark ",
                  regnal_name="MARCUS", personal_name=None),
             Pope(start=datetime.date(337, 2, 6), end=datetime.date(352, 4, 12), english_name="St. Julius I",
                  regnal_name="IULIUS", personal_name="&nbs;"),
             Pope(start=datetime.date(352, 5, 17), end=datetime.date(366, 9, 24), english_name="Liberius ",
                  regnal_name="LIBERIUS", personal_name=None),
             Pope(start=datetime.date(366, 10, 1), end=datetime.date(384, 12, 10), english_name="St. Damasus I ",
                  regnal_name="DAMASUS", personal_name=None),
             Pope(start=datetime.date(384, 12, 11), end=datetime.date(399, 11, 26), english_name="St. Siricius",
                  regnal_name="SIRICIUS", personal_name=None),
             Pope(start=datetime.date(399, 11, 27), end=datetime.date(401, 12, 19), english_name="St. Anastasius I",
                  regnal_name="ANASTASIUS", personal_name=None),
             Pope(start=datetime.date(401, 12, 22), end=datetime.date(417, 3, 12), english_name="St. Innocent I",
                  regnal_name="INNOCENTIUS", personal_name=None),
             Pope(start=datetime.date(417, 3, 18), end=datetime.date(418, 12, 26), english_name="St. Zosimus",
                  regnal_name="ZOSIMUS", personal_name=None),
             Pope(start=datetime.date(418, 12, 28), end=datetime.date(422, 9, 4), english_name="St. Boniface I",
                  regnal_name=" BONIFACIUS", personal_name=None),
             Pope(start=datetime.date(422, 9, 10), end=datetime.date(432, 7, 27), english_name="St. Celestine I",
                  regnal_name="COELESTINUS", personal_name=None),
             Pope(start=datetime.date(432, 7, 31), end=datetime.date(440, 8, 31), english_name="St. Sixtus III",
                  regnal_name="XYSTUS Tertius", personal_name=None),
             Pope(start=datetime.date(440, 9, 29), end=datetime.date(461, 11, 10),
                  english_name="St. Leo I (Leo the Great)", regnal_name="LEO MAGNUS", personal_name=None),
             Pope(start=datetime.date(461, 11, 19), end=datetime.date(468, 2, 29), english_name="St. Hilarius",
                  regnal_name="HILARIUS", personal_name=None),
             Pope(start=datetime.date(468, 3, 3), end=datetime.date(483, 3, 10), english_name="St. Simplicius",
                  regnal_name="SIMPLICIUS", personal_name=None),
             Pope(start=datetime.date(483, 3, 13), end=datetime.date(492, 3, 1),
                  english_name="St. Felix III (Felix II)", regnal_name="FELIX Tertius (Secundus)", personal_name=None),
             Pope(start=datetime.date(492, 3, 1), end=datetime.date(496, 11, 21), english_name="St. Gelasius I",
                  regnal_name="GELASIUS", personal_name=None),
             Pope(start=datetime.date(496, 11, 24), end=datetime.date(498, 11, 19), english_name="Anastasius II",
                  regnal_name="ANASTASIUS Secundus", personal_name=None),
             Pope(start=datetime.date(498, 11, 22), end=datetime.date(514, 7, 19), english_name="St. Symmachus",
                  regnal_name="SYMMACHUS", personal_name=None),
             Pope(start=datetime.date(514, 7, 19), end=datetime.date(523, 7, 19), english_name="St. Hormisdas",
                  regnal_name="HORMISDUS", personal_name=None),
             Pope(start=datetime.date(523, 8, 13), end=datetime.date(526, 5, 18), english_name="St. John I",
                  regnal_name="IOANNES", personal_name=None),
             Pope(start=datetime.date(526, 7, 13), end=datetime.date(530, 9, 22),
                  english_name="St. Felix IV (Felix III)", regnal_name="FELIX Quartus (Tertius)", personal_name=None),
             Pope(start=datetime.date(530, 9, 22), end=datetime.date(532, 10, 17), english_name="Boniface II",
                  regnal_name="BONIFACIUS Secundus", personal_name=None),
             Pope(start=datetime.date(533, 1, 2), end=datetime.date(535, 5, 8), english_name="John II",
                  regnal_name="IOANNES Secundus", personal_name="Mercurius"),
             Pope(start=datetime.date(535, 5, 13), end=datetime.date(536, 4, 22),
                  english_name="St. Agapetus I (Agapitus)", regnal_name="AGAPETUS", personal_name=None),
             Pope(start=datetime.date(536, 6, 1), end=datetime.date(537, 11, 11), english_name="St. Silverius",
                  regnal_name="SILVERIUS", personal_name=None),
             Pope(start=datetime.date(537, 3, 29), end=datetime.date(555, 6, 7), english_name="Vigilius",
                  regnal_name="VIGILIUS", personal_name=None),
             Pope(start=datetime.date(556, 4, 16), end=datetime.date(561, 3, 4), english_name="Pelagius I",
                  regnal_name=" PELAGIUS", personal_name=None),
             Pope(start=datetime.date(561, 7, 17), end=datetime.date(574, 7, 13), english_name="John III",
                  regnal_name="IOANNES Tertius", personal_name="Catelinus"),
             Pope(start=datetime.date(575, 6, 2), end=datetime.date(579, 7, 30), english_name="Benedict I",
                  regnal_name="BENEDICTUS", personal_name=None),
             Pope(start=datetime.date(579, 11, 26), end=datetime.date(590, 2, 7), english_name="Pelagius II",
                  regnal_name="PELAGIUS Secundus", personal_name=None),
             Pope(start=datetime.date(590, 9, 3), end=datetime.date(604, 3, 12),
                  english_name="St. Gregory I, O.S.B. (Gregory the Great)", regnal_name="GREGORIUS MAGNUS",
                  personal_name=None),
             Pope(start=datetime.date(604, 9, 13), end=datetime.date(606, 2, 22), english_name="Sabinian",
                  regnal_name="SABINIANUS", personal_name=None),
             Pope(start=datetime.date(607, 2, 19), end=datetime.date(607, 11, 12), english_name="Boniface III",
                  regnal_name="BONIFACIUS Tertius", personal_name=None),
             Pope(start=datetime.date(608, 8, 25), end=datetime.date(615, 5, 8), english_name="St. Boniface IV, O.S.B.",
                  regnal_name=" BONIFACIUS Quartus", personal_name=None),
             Pope(start=datetime.date(615, 10, 19), end=datetime.date(618, 11, 8),
                  english_name="Adeodatus I (Deusdedit)", regnal_name="ADEODATUS", personal_name=None),
             Pope(start=datetime.date(619, 12, 23), end=datetime.date(625, 10, 25), english_name="Boniface V",
                  regnal_name="BONIFACIUS Quintus", personal_name=None),
             Pope(start=datetime.date(625, 10, 27), end=datetime.date(638, 10, 12), english_name="Honorius I",
                  regnal_name="HONORIUS", personal_name=None),
             Pope(start=datetime.date(638, 11, 1), end=datetime.date(640, 9, 2), english_name="Severinus",
                  regnal_name="SEVERINUS", personal_name=None),
             Pope(start=datetime.date(640, 12, 24), end=datetime.date(642, 10, 12), english_name="John IV",
                  regnal_name="IOANNES Quartus", personal_name=None),
             Pope(start=datetime.date(642, 11, 24), end=datetime.date(649, 5, 14), english_name="Theodore I",
                  regnal_name="THEODORUS", personal_name=None),
             Pope(start=datetime.date(649, 8, 1), end=datetime.date(655, 10, 16), english_name="St. Martin I",
                  regnal_name="MARTINUS", personal_name=None),
             Pope(start=datetime.date(654, 8, 10), end=datetime.date(657, 6, 2), english_name="St. Eugene I",
                  regnal_name=" EUGENIUS", personal_name=None),
             Pope(start=datetime.date(657, 7, 30), end=datetime.date(672, 1, 27), english_name="St. Vitalian",
                  regnal_name="VITALIANUS", personal_name=None),
             Pope(start=datetime.date(672, 4, 11), end=datetime.date(676, 6, 17), english_name="Adeodatus II, O.S.B.",
                  regnal_name="ADEODATUS Secundus", personal_name=None),
             Pope(start=datetime.date(676, 11, 2), end=datetime.date(678, 4, 11), english_name="Donus",
                  regnal_name="DONUS", personal_name=None),
             Pope(start=datetime.date(678, 6, 27), end=datetime.date(681, 1, 10), english_name="St. Agatho",
                  regnal_name="AGATHO", personal_name=None),
             Pope(start=datetime.date(681, 12, 1), end=datetime.date(683, 7, 3), english_name="St. Leo II",
                  regnal_name="LEO Secundus", personal_name=None),
             Pope(start=datetime.date(684, 6, 26), end=datetime.date(685, 5, 8), english_name="St. Benedict II",
                  regnal_name="BENEDICTUS Secundus", personal_name=None),
             Pope(start=datetime.date(685, 7, 12), end=datetime.date(686, 8, 2), english_name="John V",
                  regnal_name="IOANNES Quintus", personal_name=None),
             Pope(start=datetime.date(686, 10, 21), end=datetime.date(687, 9, 22), english_name="Conon",
                  regnal_name="CONON", personal_name=None),
             Pope(start=datetime.date(687, 12, 15), end=datetime.date(701, 9, 8), english_name="St. Sergius I",
                  regnal_name=" SERGIUS", personal_name=None),
             Pope(start=datetime.date(701, 10, 30), end=datetime.date(705, 1, 11), english_name="John VI",
                  regnal_name="IOANNES Sextus", personal_name=None),
             Pope(start=datetime.date(705, 3, 1), end=datetime.date(707, 10, 18), english_name="John VII",
                  regnal_name="IOANNES Septimus", personal_name=None),
             Pope(start=datetime.date(708, 1, 15), end=datetime.date(708, 2, 4), english_name="Sisinnius",
                  regnal_name="SISINNIUS", personal_name=None),
             Pope(start=datetime.date(708, 3, 25), end=datetime.date(715, 4, 9), english_name="Constantine",
                  regnal_name="CONSTANTINUS", personal_name=None),
             Pope(start=datetime.date(715, 5, 19), end=datetime.date(731, 2, 11), english_name="St. Gregory II",
                  regnal_name="GREGORIUS Secundus", personal_name=None),
             Pope(start=datetime.date(731, 3, 18), end=datetime.date(741, 11, 28), english_name="Gregory III",
                  regnal_name="GREGORIUS Tertius", personal_name=None),
             Pope(start=datetime.date(741, 12, 3), end=datetime.date(752, 3, 22), english_name="St. Zachary",
                  regnal_name=" ZACHARIAS", personal_name=None),
             Pope(start=datetime.date(752, 3, 23), end=datetime.date(752, 3, 25), english_name="Pope-elect Stephen",
                  regnal_name="Electus STEPHANUS", personal_name=None),
             Pope(start=datetime.date(752, 3, 26), end=datetime.date(757, 4, 26),
                  english_name="Stephen II (Stephen III)", regnal_name="STEPHANUS Secundus (Tertius)",
                  personal_name=None),
             Pope(start=datetime.date(757, 5, 29), end=datetime.date(767, 6, 28), english_name="St. Paul I",
                  regnal_name=" PAULUS", personal_name=None),
             Pope(start=datetime.date(767, 8, 1), end=datetime.date(772, 1, 24),
                  english_name="Stephen III (Stephen IV)", regnal_name="STEPHANUS Tertius (Quartus)",
                  personal_name=None),
             Pope(start=datetime.date(772, 2, 1), end=datetime.date(795, 12, 26), english_name="Adrian I",
                  regnal_name="HADRIANUS", personal_name=None),
             Pope(start=datetime.date(795, 12, 26), end=datetime.date(816, 6, 12), english_name="St. Leo III",
                  regnal_name="LEO Tertius", personal_name=None),
             Pope(start=datetime.date(816, 6, 12), end=datetime.date(817, 1, 24), english_name="Stephen IV (Stephen V)",
                  regnal_name="STEPHANUS Quartus (Quintus)", personal_name=None),
             Pope(start=datetime.date(817, 1, 25), end=datetime.date(824, 2, 11), english_name="St. Paschal I",
                  regnal_name="PASCHALIS", personal_name=None),
             Pope(start=datetime.date(824, 5, 8), end=datetime.date(827, 7, 31), english_name="Eugene II",
                  regnal_name="EUGENIUS Secundus", personal_name=None),
             Pope(start=datetime.date(827, 8, 1), end=datetime.date(827, 8, 30), english_name="Valentine",
                  regnal_name="VALENTINUS", personal_name=None),
             Pope(start=datetime.date(827, 9, 1), end=datetime.date(843, 12, 31), english_name="Gregory IV",
                  regnal_name="GREGORIUS Quartus", personal_name=None),
             Pope(start=datetime.date(844, 1, 1), end=datetime.date(847, 1, 7), english_name="Sergius II",
                  regnal_name="SERGIUS Secundus", personal_name=None),
             Pope(start=datetime.date(847, 1, 8), end=datetime.date(855, 7, 17), english_name="St. Leo IV, O.S.B.",
                  regnal_name="LEO Quartus", personal_name=None),
             Pope(start=datetime.date(855, 7, 18), end=datetime.date(891, 9, 14), english_name="Benedict III",
                  regnal_name="BENEDICTUS Tertius", personal_name=None),
             Pope(start=datetime.date(858, 4, 24), end=datetime.date(867, 11, 13),
                  english_name="St. Nicholas I (Nicholas the Great)", regnal_name="NICOLAUS MAGNUS",
                  personal_name=None),
             Pope(start=datetime.date(867, 12, 14), end=datetime.date(872, 12, 14), english_name="Adrian II",
                  regnal_name="HADRIANUS Secundus", personal_name=None),
             Pope(start=datetime.date(872, 12, 14), end=datetime.date(882, 12, 16), english_name="John VIII",
                  regnal_name="IOANNES Octavus", personal_name=None),
             Pope(start=datetime.date(882, 12, 16), end=datetime.date(884, 5, 15), english_name="Marinus I",
                  regnal_name=" MARINUS", personal_name=None),
             Pope(start=datetime.date(884, 5, 17), end=datetime.date(884, 12, 31), english_name="St. Adrian III",
                  regnal_name="HADRIANUS Tertius", personal_name=None),
             Pope(start=datetime.date(885, 1, 1), end=datetime.date(891, 9, 14), english_name="Stephen V (Stephen VI)",
                  regnal_name="STEPHANUS Quintus (Sextus)", personal_name=None),
             Pope(start=datetime.date(891, 9, 19), end=datetime.date(896, 4, 4), english_name="Formosus",
                  regnal_name="FORMOSUS", personal_name=None),
             Pope(start=datetime.date(896, 4, 4), end=datetime.date(896, 4, 19), english_name="Boniface VI",
                  regnal_name="BONIFACIUS Sextus", personal_name=None),
             Pope(start=datetime.date(896, 5, 22), end=datetime.date(897, 7, 31),
                  english_name="Stephen VI (Stephen VII)", regnal_name="STEPHANUS Sextus (Septimus)",
                  personal_name=None),
             Pope(start=datetime.date(897, 8, 1), end=datetime.date(897, 10, 31), english_name="Romanus",
                  regnal_name="ROMANUS", personal_name=None),
             Pope(start=datetime.date(897, 11, 1), end=datetime.date(897, 12, 31), english_name="Theodore II",
                  regnal_name="THEODORUS Secundus", personal_name=None),
             Pope(start=datetime.date(898, 1, 1), end=datetime.date(899, 12, 31), english_name="John IX, O.S.B.",
                  regnal_name="IOANNES Nonus", personal_name=None),
             Pope(start=datetime.date(900, 1, 1), end=datetime.date(903, 6, 30), english_name="Benedict IV",
                  regnal_name="BENEDICTUS Quartus", personal_name=None),
             Pope(start=datetime.date(903, 7, 1), end=datetime.date(903, 9, 30), english_name="Leo V",
                  regnal_name=" LEO Quintus", personal_name=None),
             Pope(start=datetime.date(904, 1, 29), end=datetime.date(911, 4, 14), english_name="Sergius III",
                  regnal_name="SERGIUS Tertius", personal_name=None),
             Pope(start=datetime.date(911, 4, 15), end=datetime.date(913, 6, 30), english_name="Anastasius III",
                  regnal_name="ANASTASIUS Tertius", personal_name=None),
             Pope(start=datetime.date(913, 7, 1), end=datetime.date(914, 2, 28), english_name="Lando",
                  regnal_name="LANDO", personal_name=None),
             Pope(start=datetime.date(914, 3, 1), end=datetime.date(928, 4, 30), english_name="John X",
                  regnal_name="IOANNES Decimus", personal_name=None),
             Pope(start=datetime.date(928, 5, 1), end=datetime.date(928, 11, 30), english_name="Leo VI",
                  regnal_name="LEO Sextus", personal_name=None),
             Pope(start=datetime.date(928, 12, 1), end=datetime.date(931, 2, 28),
                  english_name="Stephen VII (Stephen VIII)", regnal_name="STEPHANUS Septimus (Octavus)",
                  personal_name=None),
             Pope(start=datetime.date(931, 3, 1), end=datetime.date(935, 12, 31), english_name="John XI",
                  regnal_name=" IOANNES Undecimus", personal_name=None),
             Pope(start=datetime.date(936, 1, 3), end=datetime.date(939, 7, 13), english_name="Leo VII, O.S.B.",
                  regnal_name="LEO Septimus", personal_name=None),
             Pope(start=datetime.date(939, 7, 14), end=datetime.date(942, 10, 29),
                  english_name="Stephen VIII (Stephen IX)", regnal_name="STEPHANUS Octavus (Nonus)",
                  personal_name=None),
             Pope(start=datetime.date(942, 10, 30), end=datetime.date(946, 5, 9), english_name="Marinus II",
                  regnal_name="MARINUS Secundus", personal_name=None),
             Pope(start=datetime.date(946, 5, 10), end=datetime.date(955, 12, 15), english_name="Agapetus II",
                  regnal_name="AGAPETUS Secundus", personal_name=None),
             Pope(start=datetime.date(955, 12, 16), end=datetime.date(964, 5, 14), english_name="John XII",
                  regnal_name="IOANNES Duodecimus", personal_name="Octavian"),
             Pope(start=datetime.date(964, 5, 22), end=datetime.date(964, 6, 23), english_name="Benedict V",
                  regnal_name="BENEDICTUS Quintus", personal_name=None),
             Pope(start=datetime.date(964, 7, 1), end=datetime.date(965, 3, 1), english_name="Leo VIII",
                  regnal_name="LEO Octavus", personal_name=None),
             Pope(start=datetime.date(965, 10, 1), end=datetime.date(972, 9, 6), english_name="John XIII",
                  regnal_name="IOANNES Tertius Decimus", personal_name=None),
             Pope(start=datetime.date(973, 1, 19), end=datetime.date(974, 6, 30), english_name="Benedict VI",
                  regnal_name=" BENEDICTUS Sextus", personal_name=None),
             Pope(start=datetime.date(974, 10, 1), end=datetime.date(983, 7, 10), english_name="Benedict VII",
                  regnal_name=" BENEDICTUS Septimus", personal_name=None),
             Pope(start=datetime.date(983, 12, 1), end=datetime.date(984, 8, 20), english_name="John XIV",
                  regnal_name="IOANNES Quartus Decimus", personal_name="Pietro Campanora"),
             Pope(start=datetime.date(985, 8, 21), end=datetime.date(996, 3, 31), english_name="John XV",
                  regnal_name="IOANNES Quintus Decimus", personal_name=" "),
             Pope(start=datetime.date(996, 5, 3), end=datetime.date(999, 2, 18), english_name="Gregory V",
                  regnal_name="GREGORIUS Quintus", personal_name="Bruno of Carinthia"),
             Pope(start=datetime.date(999, 4, 2), end=datetime.date(1003, 5, 12), english_name="Sylvester II",
                  regnal_name="SILVESTER Secundus", personal_name="Gerbert d'Aurillac"),
             Pope(start=datetime.date(1003, 6, 1), end=datetime.date(1003, 12, 24), english_name="John XVII",
                  regnal_name=" IOANNES Septimus Decimus", personal_name="Siccone"),
             Pope(start=datetime.date(1003, 12, 25), end=datetime.date(1009, 7, 30), english_name="John XVIII",
                  regnal_name="IOANNES DuodeVicesimus", personal_name="Giovanni Fasano; Phasianus"),
             Pope(start=datetime.date(1009, 7, 31), end=datetime.date(1012, 5, 12), english_name="Sergius IV",
                  regnal_name="SERGIUS Quartus", personal_name="Pietro Boccapecora"),
             Pope(start=datetime.date(1012, 5, 18), end=datetime.date(1024, 4, 9), english_name="Benedict VIII",
                  regnal_name="BENEDICTUS Octavus", personal_name="Theophylactus II, Conti di Tusculum"),
             Pope(start=datetime.date(1024, 4, 1), end=datetime.date(1032, 10, 20), english_name="John XIX",
                  regnal_name="IOANNES UndeVicesimus", personal_name="Romanus, Conti di Tusculum"),
             Pope(start=datetime.date(1032, 10, 21), end=datetime.date(1043, 12, 31), english_name="Benedict IX",
                  regnal_name="BENEDICTUS Nonus", personal_name="Theophylactus III, Conti di Tusculum"),
             Pope(start=datetime.date(1044, 1, 1), end=datetime.date(1044, 12, 31), english_name="Sylvester III",
                  regnal_name="SILVESTER Tertius", personal_name="John, Bishop of Sabina"),
             Pope(start=datetime.date(1045, 1, 1), end=datetime.date(1045, 3, 30), english_name="Benedict IX",
                  regnal_name="BENEDICTUS Nonus", personal_name="Theophylactus III, Conti di Tusculum"),
             Pope(start=datetime.date(1045, 4, 1), end=datetime.date(1046, 12, 20), english_name="Gregory VI",
                  regnal_name="GREGORIUS Sextus", personal_name="Johannes Gratianus"),
             Pope(start=datetime.date(1046, 12, 24), end=datetime.date(1047, 10, 9), english_name="Clement II",
                  regnal_name="CLEMENS Secundus", personal_name="Suidger"),
             Pope(start=datetime.date(1047, 11, 1), end=datetime.date(1048, 7, 16), english_name="Benedict IX",
                  regnal_name="BENEDICTUS Nonus", personal_name="Theophylactus III, Conti di Tusculum"),
             Pope(start=datetime.date(1048, 7, 17), end=datetime.date(1048, 8, 9), english_name="Damasus II",
                  regnal_name="DAMASUS Secundus", personal_name="Poppo"),
             Pope(start=datetime.date(1049, 2, 12), end=datetime.date(1054, 4, 19), english_name="St. Leo IX",
                  regnal_name="LEO Nonus", personal_name="Bruno, Count of Dagsbourg"),
             Pope(start=datetime.date(1055, 4, 13), end=datetime.date(1057, 7, 28), english_name="Victor II",
                  regnal_name="VICTOR Secundus", personal_name="Gebhard, Count of Calw, Tollenstein, and Hirschberg"),
             Pope(start=datetime.date(1057, 8, 2), end=datetime.date(1058, 3, 29),
                  english_name="Stephen IX (Stephen X), O.S.B.", regnal_name="STEPHANUS Nonus (Decimus)",
                  personal_name="Frederic de Lorraine"),
             Pope(start=datetime.date(1058, 12, 6), end=datetime.date(1061, 7, 27), english_name="Nicholas II",
                  regnal_name="NICOLAUS Secundus", personal_name="Gerard de Bourgogne"),
             Pope(start=datetime.date(1061, 9, 30), end=datetime.date(1073, 4, 21), english_name="Alexander II",
                  regnal_name="ALEXANDER Secundus", personal_name="Anselmo da Baggio"),
             Pope(start=datetime.date(1073, 4, 22), end=datetime.date(1085, 5, 25),
                  english_name="St. Gregory VII, O.S.B.", regnal_name="GREGORIUS Septimus", personal_name="Hildebrand"),
             Pope(start=datetime.date(1086, 5, 24), end=datetime.date(1087, 9, 16),
                  english_name="Bd. Victor III, O.S.B.", regnal_name="VICTOR Tertius",
                  personal_name="Desiderio; Desiderius; Dauferius"),
             Pope(start=datetime.date(1088, 3, 12), end=datetime.date(1099, 7, 29), english_name="Bd. Urban II, O.S.B.",
                  regnal_name="URBANUS Secundus", personal_name="Odo of Lagery"),
             Pope(start=datetime.date(1099, 8, 13), end=datetime.date(1118, 1, 21), english_name="Paschal II, O.S.B.",
                  regnal_name="PASCHALIS Secundus", personal_name="Raniero"),
             Pope(start=datetime.date(1118, 1, 24), end=datetime.date(1119, 1, 28), english_name="Gelasius II, O.S.B.",
                  regnal_name="GELASIUS Secundus", personal_name="Giovanni Coniulo"),
             Pope(start=datetime.date(1119, 2, 2), end=datetime.date(1124, 12, 13), english_name="Callixtus II",
                  regnal_name="CALLISTUS Secundus", personal_name="Guido, Comte de Bourgogne"),
             Pope(start=datetime.date(1124, 12, 15), end=datetime.date(1130, 2, 13),
                  english_name="Honorius II, Can.Reg.", regnal_name="HONORIUS Secundus",
                  personal_name="Lamberto Scannabecchi"),
             Pope(start=datetime.date(1130, 2, 14), end=datetime.date(1143, 9, 24),
                  english_name="Innocent II, Can.Reg.", regnal_name="INNOCENTIUS Secundus",
                  personal_name="Gregorio Papareschi"),
             Pope(start=datetime.date(1143, 9, 26), end=datetime.date(1144, 3, 8), english_name="Celestine II",
                  regnal_name="COELESTINUS Secundus", personal_name="Guido"),
             Pope(start=datetime.date(1144, 3, 12), end=datetime.date(1145, 2, 15), english_name="Lucius II, Can.Reg.",
                  regnal_name="LUCIUS Secundus", personal_name="Gerardo Caccianemici dal Orso"),
             Pope(start=datetime.date(1145, 2, 15), end=datetime.date(1153, 7, 8),
                  english_name="Bd. Eugene III, O.Cist.", regnal_name="EUGENIUS Tertius",
                  personal_name="Bernardo da Pisa"),
             Pope(start=datetime.date(1153, 7, 8), end=datetime.date(1154, 12, 3), english_name="Anastasius IV",
                  regnal_name="ANASTASIUS Quartus", personal_name="Corrado Demetri della Suburra"),
             Pope(start=datetime.date(1154, 12, 4), end=datetime.date(1159, 9, 1), english_name="Adrian IV, O.S.A.",
                  regnal_name="HADRIANUS Quartus", personal_name="Nicholas Breakspear"),
             Pope(start=datetime.date(1159, 9, 7), end=datetime.date(1181, 8, 30), english_name="Alexander III",
                  regnal_name=" ALEXANDER Tertius", personal_name="Rolando"),
             Pope(start=datetime.date(1181, 9, 1), end=datetime.date(1185, 11, 25), english_name="Lucius III",
                  regnal_name="LUCIUS Tertius", personal_name="Ubaldo"),
             Pope(start=datetime.date(1185, 11, 25), end=datetime.date(1187, 10, 19), english_name="Urban III",
                  regnal_name="URBANUS Tertius", personal_name="Uberto Crivelli"),
             Pope(start=datetime.date(1187, 10, 21), end=datetime.date(1187, 12, 17),
                  english_name="Gregory VIII, Can.Reg.", regnal_name=" GREGORIUS Octavus",
                  personal_name="Alberto di Morra"),
             Pope(start=datetime.date(1187, 12, 19), end=datetime.date(1191, 3, 20), english_name="Clement III",
                  regnal_name=" CLEMENS Tertius", personal_name="Paolo Scolari"),
             Pope(start=datetime.date(1191, 3, 21), end=datetime.date(1198, 1, 8), english_name="Celestine III",
                  regnal_name="COELESTINUS Tertius", personal_name="Giacinto Bobone"),
             Pope(start=datetime.date(1198, 1, 8), end=datetime.date(1216, 7, 16), english_name="Innocent III",
                  regnal_name="INNOCENTIUS Tertius", personal_name="Lothario dei Conti di Segni"),
             Pope(start=datetime.date(1216, 7, 18), end=datetime.date(1227, 3, 18), english_name="Honorius III",
                  regnal_name="HONORIUS Tertius", personal_name="Cencio"),
             Pope(start=datetime.date(1227, 3, 19), end=datetime.date(1241, 8, 22), english_name="Gregory IX",
                  regnal_name="GREGORIUS Nonus", personal_name="Ugolino dei Conti di Segni"),
             Pope(start=datetime.date(1241, 10, 25), end=datetime.date(1241, 11, 10), english_name="Celestine IV",
                  regnal_name="COELESTINUS Quartus", personal_name="Goffredo Castiglioni"),
             Pope(start=datetime.date(1243, 6, 25), end=datetime.date(1254, 12, 7), english_name="Innocent IV",
                  regnal_name="INNOCENTIUS Quartus", personal_name="Sinibaldo Fieschi"),
             Pope(start=datetime.date(1254, 12, 12), end=datetime.date(1261, 5, 25), english_name="Alexander IV",
                  regnal_name="ALEXANDER Quartus", personal_name="Rinaldo dei Conti di Jenne"),
             Pope(start=datetime.date(1261, 8, 29), end=datetime.date(1264, 10, 2), english_name="Urban IV",
                  regnal_name="URBANUS Quartus", personal_name="Jacques Pantaleon"),
             Pope(start=datetime.date(1265, 2, 5), end=datetime.date(1268, 11, 29), english_name="Clement IV",
                  regnal_name="CLEMENS Quartus", personal_name="Gui Faucoi"),
             Pope(start=datetime.date(1268, 11, 29), end=datetime.date(1271, 9, 1), english_name="nan",
                  regnal_name="nan", personal_name=" "),
             Pope(start=datetime.date(1271, 9, 1), end=datetime.date(1276, 1, 10), english_name="Bd. Gregory X",
                  regnal_name="GREGORIUS Decimus", personal_name="Tebaldo Visconti"),
             Pope(start=datetime.date(1276, 1, 21), end=datetime.date(1276, 6, 22), english_name="Bd. Innocent V, O.P.",
                  regnal_name="INNOCENTIUS Quintus", personal_name="Pierre de Tarentaise"),
             Pope(start=datetime.date(1276, 7, 11), end=datetime.date(1276, 8, 18), english_name="Adrian V",
                  regnal_name="HADRIANUS Quintus", personal_name="Ottobuono Fieschi"),
             Pope(start=datetime.date(1276, 9, 8), end=datetime.date(1277, 5, 20), english_name="John XXI ",
                  regnal_name="IOANNES Vicesimus Primus", personal_name="Pedro Hispano"),
             Pope(start=datetime.date(1277, 11, 25), end=datetime.date(1280, 8, 22), english_name="Nicholas III ",
                  regnal_name="NICOLAUS Tertius", personal_name="Giovanni Gaetano Orsini"),
             Pope(start=datetime.date(1281, 2, 22), end=datetime.date(1285, 3, 28), english_name="Martin IV",
                  regnal_name="MARTINUS Quartus", personal_name="Simon de Brion"),
             Pope(start=datetime.date(1285, 4, 2), end=datetime.date(1287, 4, 3), english_name="Honorius IV",
                  regnal_name="HONORIUS Quartus", personal_name="Giacomo Savelli"),
             Pope(start=datetime.date(1288, 2, 22), end=datetime.date(1292, 4, 4), english_name="Nicholas IV, O.F.M.",
                  regnal_name="NICOLAUS Quartus", personal_name="Girolamo Masci"),
             Pope(start=datetime.date(1292, 4, 4), end=datetime.date(1294, 7, 5), english_name="interregnum",
                  regnal_name="nan", personal_name=" "),
             Pope(start=datetime.date(1294, 7, 5), end=datetime.date(1294, 12, 13),
                  english_name="St. Celestine V, O.S.B.", regnal_name="COELESTINUS Quintus",
                  personal_name="Pietro da Morrone"),
             Pope(start=datetime.date(1294, 12, 24), end=datetime.date(1303, 10, 11), english_name="Boniface VIII",
                  regnal_name="BONIFACIUS Octavus", personal_name="Benedetto Caetani"),
             Pope(start=datetime.date(1303, 10, 22), end=datetime.date(1304, 7, 7),
                  english_name="Bd. Benedict XI, O.P.", regnal_name="BENEDICTUS Undecimus",
                  personal_name="Niccolo Boccasini"),
             Pope(start=datetime.date(1305, 6, 5), end=datetime.date(1314, 4, 20), english_name="Clement V",
                  regnal_name="CLEMENS Quintus", personal_name="Bertrand de Got"),
             Pope(start=datetime.date(1314, 4, 20), end=datetime.date(1316, 8, 7), english_name="interregnum",
                  regnal_name=" ", personal_name=" "),
             Pope(start=datetime.date(1316, 8, 7), end=datetime.date(1334, 12, 4), english_name="John XXII",
                  regnal_name="IOANNES Vicesimus Secundus", personal_name="Jacques d'Euse; Jacques Duese"),
             Pope(start=datetime.date(1334, 12, 20), end=datetime.date(1342, 4, 25),
                  english_name="Benedict XII, O.Cist.", regnal_name=" BENEDICTUS Duodecimus",
                  personal_name="Jacques Fournier"),
             Pope(start=datetime.date(1342, 5, 7), end=datetime.date(1352, 12, 6), english_name="Clement VI, O.S.B.",
                  regnal_name="CLEMENS Sextus", personal_name="Pierre Roger"),
             Pope(start=datetime.date(1352, 12, 18), end=datetime.date(1362, 9, 12), english_name="Innocent VI",
                  regnal_name="INNOCENTIUS Sextus", personal_name="Etienne Aubert"),
             Pope(start=datetime.date(1362, 9, 28), end=datetime.date(1370, 12, 19), english_name="Bd. Urban V, O.S.B.",
                  regnal_name="URBANUS Quintus", personal_name="Guillaume (de) Grimoard"),
             Pope(start=datetime.date(1378, 4, 8), end=datetime.date(1389, 10, 15), english_name="Urban VI",
                  regnal_name="URBANUS Sextus", personal_name="Bartolomeo Prignano"),
             Pope(start=datetime.date(1389, 11, 2), end=datetime.date(1404, 10, 1), english_name="Boniface IX",
                  regnal_name="BONIFACIUS Nonus", personal_name="Pietro Tomacelli"),
             Pope(start=datetime.date(1404, 10, 17), end=datetime.date(1406, 11, 6), english_name="Innocent VII",
                  regnal_name="INNOCENTIUS Septimus", personal_name="Cosimo Gentile Migliorati"),
             Pope(start=datetime.date(1406, 11, 30), end=datetime.date(1415, 7, 4), english_name="nan",
                  regnal_name="GREGORIUS Duodecimus", personal_name="Angelo Correr"),
             Pope(start=datetime.date(1415, 7, 4), end=datetime.date(1417, 11, 11), english_name=" ", regnal_name=" ",
                  personal_name=" "),
             Pope(start=datetime.date(1417, 11, 11), end=datetime.date(1431, 2, 20), english_name="Martin V",
                  regnal_name="MARTINUS Quintus", personal_name="Oddone Colonna"),
             Pope(start=datetime.date(1431, 3, 3), end=datetime.date(1447, 2, 23), english_name="Eugene IV, O.S.A.",
                  regnal_name="EUGENIUS Quartus", personal_name="Gabriele Condulmer"),
             Pope(start=datetime.date(1447, 3, 6), end=datetime.date(1455, 3, 24), english_name="Nicholas V",
                  regnal_name="NICOLAUS Quintus", personal_name="Tommaso Parentucelli"),
             Pope(start=datetime.date(1455, 4, 8), end=datetime.date(1458, 8, 6), english_name="Callixtus III",
                  regnal_name="CALLISTUS Tertius", personal_name="Alfonso de Borgia"),
             Pope(start=datetime.date(1458, 8, 19), end=datetime.date(1464, 8, 15), english_name="Pius II",
                  regnal_name="PIUS Secundus", personal_name="Enea Silvio Piccolomini"),
             Pope(start=datetime.date(1464, 8, 30), end=datetime.date(1471, 7, 26), english_name="Paul II",
                  regnal_name="PAULUS Secundus", personal_name="Pietro Barbo"),
             Pope(start=datetime.date(1471, 8, 9), end=datetime.date(1484, 8, 12), english_name="Sixtus IV, O.F.M.",
                  regnal_name="XYSTUS Quartus", personal_name="Francesco della Rovere"),
             Pope(start=datetime.date(1484, 8, 29), end=datetime.date(1492, 7, 25), english_name="Innocent VIII",
                  regnal_name="INNOCENTIUS Octavus", personal_name="Giovanni Battista Cybo"),
             Pope(start=datetime.date(1492, 8, 11), end=datetime.date(1503, 8, 18), english_name="Alexander VI",
                  regnal_name="ALEXANDER Sextus", personal_name="Rodrigo de Lanzol-Borgia"),
             Pope(start=datetime.date(1503, 9, 22), end=datetime.date(1503, 10, 18), english_name="Pius III ",
                  regnal_name="PIUS Tertius", personal_name="Francesco Todeschini Piccolomini"),
             Pope(start=datetime.date(1503, 10, 31), end=datetime.date(1513, 2, 21), english_name="Julius II ",
                  regnal_name="IULIUS Secundus", personal_name="Giuliano della Rovere"),
             Pope(start=datetime.date(1513, 3, 9), end=datetime.date(1521, 12, 1), english_name="Leo X",
                  regnal_name="LEO Decimus", personal_name="Giovanni di Lorenzo de' Medici"),
             Pope(start=datetime.date(1522, 1, 9), end=datetime.date(1523, 9, 14),
                  english_name="Adrian VI Papa HADRIANUS Sextus", regnal_name="HADRIANUS Sextus",
                  personal_name="Adriaan Floriszoon Boeyens"),
             Pope(start=datetime.date(1523, 11, 26), end=datetime.date(1534, 9, 25), english_name="Clement VII",
                  regnal_name="CLEMENS Septimus", personal_name="Giulio di Giuliano de' Medici"),
             Pope(start=datetime.date(1534, 10, 13), end=datetime.date(1549, 11, 10), english_name="Paul III",
                  regnal_name="PAULUS Tertius", personal_name="Alessandro Farnese"),
             Pope(start=datetime.date(1550, 2, 7), end=datetime.date(1555, 3, 29), english_name="Julius III",
                  regnal_name="IULIUS Tertius", personal_name="Giovanni Maria Ciocchi del Monte"),
             Pope(start=datetime.date(1555, 4, 9), end=datetime.date(1555, 5, 1), english_name="Marcellus II",
                  regnal_name="MARCELLUS Secundus", personal_name="Marcello Cervini"),
             Pope(start=datetime.date(1555, 5, 23), end=datetime.date(1559, 8, 18), english_name="Paul IV, C.R.",
                  regnal_name="PAULUS Quartus", personal_name="Giovanni Pietro Carafa"),
             Pope(start=datetime.date(1559, 12, 26), end=datetime.date(1565, 12, 9), english_name="Pius IV",
                  regnal_name="PIUS Quartus", personal_name="Giovanni Angelo Medici"),
             Pope(start=datetime.date(1566, 1, 7), end=datetime.date(1572, 5, 1), english_name="St. Pius V, O.P.",
                  regnal_name="PIUS Quintus", personal_name="Michele Ghislieri"),
             Pope(start=datetime.date(1572, 5, 13), end=datetime.date(1585, 4, 10), english_name="Gregory XIII",
                  regnal_name="GREGORIUS Tertius Decimus", personal_name="Ugo Boncompagni"),
             Pope(start=datetime.date(1585, 4, 24), end=datetime.date(1590, 8, 27),
                  english_name="Sixtus V, O.F.M. Conv.", regnal_name="XYSTUS Quintus", personal_name="Felice Peretti"),
             Pope(start=datetime.date(1590, 9, 15), end=datetime.date(1590, 9, 27), english_name="Urban VII",
                  regnal_name="URBANUS Septimus", personal_name="Giovanni Battista Castagna"),
             Pope(start=datetime.date(1590, 12, 5), end=datetime.date(1591, 10, 16), english_name="Gregory XIV",
                  regnal_name="GREGORIUS Quartus Decimus", personal_name="Niccolo Sfondrati"),
             Pope(start=datetime.date(1591, 10, 29), end=datetime.date(1591, 12, 30), english_name="Innocent IX",
                  regnal_name="INNOCENTIUS Nonus", personal_name="Giovanni Antonio Facchinetti"),
             Pope(start=datetime.date(1592, 1, 30), end=datetime.date(1605, 3, 3), english_name="Clement VIII",
                  regnal_name="CLEMENS Octavus", personal_name="Ippolito Aldobrandini"),
             Pope(start=datetime.date(1605, 4, 1), end=datetime.date(1605, 4, 27), english_name="Leo XI ",
                  regnal_name="LEO Undecimus", personal_name="Alessandro Ottaviano de Medici"),
             Pope(start=datetime.date(1605, 5, 16), end=datetime.date(1621, 1, 28), english_name="Paul V",
                  regnal_name="PAULUS Quintus", personal_name="Camillo Borghese"),
             Pope(start=datetime.date(1621, 2, 9), end=datetime.date(1623, 7, 8), english_name="Gregory XV",
                  regnal_name="GREGORIUS Quintus Decimus", personal_name="Alessandro Ludovisi"),
             Pope(start=datetime.date(1623, 8, 6), end=datetime.date(1644, 7, 29), english_name="Urban VIII",
                  regnal_name="URBANUS Octavus", personal_name="Maffeo Barberini"),
             Pope(start=datetime.date(1644, 9, 15), end=datetime.date(1655, 1, 7), english_name="Innocent X",
                  regnal_name="INNOCENTIUS Decimus", personal_name="Giovanni Battista Pamphilj"),
             Pope(start=datetime.date(1655, 4, 7), end=datetime.date(1667, 5, 22), english_name="Alexander VII",
                  regnal_name="ALEXANDER Septimus", personal_name="Fabio Chigi"),
             Pope(start=datetime.date(1667, 6, 20), end=datetime.date(1669, 12, 9), english_name="Clement IX",
                  regnal_name="CLEMENS Nonus", personal_name="Giulio Rospigliosi"),
             Pope(start=datetime.date(1670, 4, 29), end=datetime.date(1676, 7, 22), english_name="Clement X",
                  regnal_name="CLEMENS Decimus", personal_name="Emilio Altieri"),
             Pope(start=datetime.date(1676, 9, 21), end=datetime.date(1689, 8, 12), english_name="Bd. Innocent XI",
                  regnal_name="INNOCENTIUS Undecimus", personal_name="Benedetto Odescalchi"),
             Pope(start=datetime.date(1689, 10, 6), end=datetime.date(1691, 2, 1), english_name="Alexander VIII",
                  regnal_name="ALEXANDER Octavus", personal_name="Pietro Vito Ottoboni"),
             Pope(start=datetime.date(1691, 7, 12), end=datetime.date(1700, 9, 27), english_name="Innocent XII",
                  regnal_name="INNOCENTIUS Duodecimus", personal_name="Antonio Pignatelli"),
             Pope(start=datetime.date(1700, 11, 23), end=datetime.date(1721, 3, 19), english_name="Clement XI",
                  regnal_name="CLEMENS Undecimus", personal_name="Giovanni Francesco Albani"),
             Pope(start=datetime.date(1721, 5, 8), end=datetime.date(1724, 3, 7), english_name="Innocent XIII",
                  regnal_name="Tertius Decimus", personal_name="Michelangelo de Conti"),
             Pope(start=datetime.date(1724, 5, 29), end=datetime.date(1730, 2, 21), english_name="Benedict XIII, O.P.",
                  regnal_name=" BENEDICTUS Tertius Decimus", personal_name="Pierfrancesco Orsini"),
             Pope(start=datetime.date(1730, 7, 12), end=datetime.date(1740, 2, 6), english_name="Clement XII",
                  regnal_name="CLEMENS Duodecimus", personal_name="Lorenzo Corsini"),
             Pope(start=datetime.date(1740, 8, 17), end=datetime.date(1758, 5, 3), english_name="Benedict XIV",
                  regnal_name="BENEDICTUS Quartus Decimus", personal_name="Prospero Lorenzo Lambertini"),
             Pope(start=datetime.date(1758, 7, 6), end=datetime.date(1769, 2, 2), english_name="Clement XIII",
                  regnal_name="CLEMENS Tertius Decimus", personal_name="Carlo della Torre Rezzonico"),
             Pope(start=datetime.date(1769, 5, 19), end=datetime.date(1774, 9, 22), english_name="Clement XIV, O.F.M.",
                  regnal_name="CLEMENS Quartus Decimus", personal_name="Giovanni Vincenzo Antonio Ganganelli"),
             Pope(start=datetime.date(1775, 2, 15), end=datetime.date(1799, 8, 29), english_name="Pius VI",
                  regnal_name="PIUS Sextus", personal_name="Count Giovanni Angelo Braschi"),
             Pope(start=datetime.date(1800, 3, 14), end=datetime.date(1823, 8, 20), english_name="Pius VII, O.S.B.",
                  regnal_name="PIUS Septimus", personal_name="Barnaba Chiaramonti"),
             Pope(start=datetime.date(1823, 9, 28), end=datetime.date(1829, 2, 10), english_name="Leo XII",
                  regnal_name="LEO Duodecimus", personal_name="Count Annibale Sermattei della Genga"),
             Pope(start=datetime.date(1829, 3, 31), end=datetime.date(1830, 12, 1), english_name="Pius VIII",
                  regnal_name="PIUS Octavus", personal_name="Francesco Saverio Castiglioni"),
             Pope(start=datetime.date(1831, 2, 2), end=datetime.date(1846, 6, 1),
                  english_name="Gregory XVI, O.S.B. Cam.", regnal_name="GREGORIUS Sextus Decimus",
                  personal_name="Bartolomeo Alberto Cappellari"),
             Pope(start=datetime.date(1846, 6, 16), end=datetime.date(1878, 2, 7), english_name="Bd. Pius IX, O.F.S.",
                  regnal_name="PIUS Nonus", personal_name="Count Giovanni Maria Mastai-Ferretti"),
             Pope(start=datetime.date(1878, 2, 20), end=datetime.date(1903, 7, 20), english_name="Leo XIII, O.F.S.",
                  regnal_name="LEO Tertius Decimus", personal_name="Gioacchino Vincenzo Raffaele Luigi Pecci"),
             Pope(start=datetime.date(1903, 8, 4), end=datetime.date(1914, 8, 20), english_name="St. Pius X",
                  regnal_name="PIUS Decimus", personal_name="Giuseppe Melchiorre Sarto"),
             Pope(start=datetime.date(1914, 9, 3), end=datetime.date(1922, 1, 22), english_name="Benedict XV",
                  regnal_name="BENEDICTUS Quintus Decimus", personal_name="Giacomo Della Chiesa"),
             Pope(start=datetime.date(1922, 2, 6), end=datetime.date(1939, 2, 10), english_name="Pius XI",
                  regnal_name="PIUS Undecimus", personal_name="Achille Ambrogio Damiano Ratti"),
             Pope(start=datetime.date(1939, 3, 2), end=datetime.date(1958, 10, 9), english_name="Ven. Pius XII",
                  regnal_name="PIUS Duodecimus", personal_name="Eugenio Maria Giuseppe Giovanni Pacelli"),
             Pope(start=datetime.date(1958, 10, 28), end=datetime.date(1963, 6, 3), english_name="Bd. John XXIII",
                  regnal_name="IOANNES Vicesimus Tertius", personal_name="Angelo Giuseppe Roncalli"),
             Pope(start=datetime.date(1963, 6, 21), end=datetime.date(1978, 8, 6),
                  english_name="Servant of God Paul VI", regnal_name="PAULUS Sextus",
                  personal_name="Giovanni Battista Enrico Antonio Maria Montini"),
             Pope(start=datetime.date(1978, 8, 26), end=datetime.date(1978, 9, 28),
                  english_name="Servant of God John Paul I", regnal_name="IOANNES PAULUS Primus",
                  personal_name="Albino Luciani"),
             Pope(start=datetime.date(1978, 10, 16), end=datetime.date(2005, 4, 2),
                  english_name="Ven. John Paul II (John Paul the Great)", regnal_name="IOANNES PAULUS Secundus",
                  personal_name="Karol Jozef Wojtyla"),
             Pope(start=datetime.date(2005, 4, 19), end=datetime.date(2013, 2, 28), english_name="Benedict XVI",
                  regnal_name="BENEDICTUS Sextus Decimus", personal_name="Joseph Alois Ratzinger"),
             Pope(start=datetime.date(2013, 3, 13), end=None, english_name="Francis I", regnal_name="Franciscum",
                  personal_name="Jorge Mario Bergoglio"), ]

    @classmethod
    def who(cls, dt):
        """
        Returns first matched popet.Pope object with popet.Pope.in_inauguration(dt)
        """
        ind = bisect_right(cls.popes, dt)
        if ind and cls.popes[ind - 1].in_inauguration(dt):
            return cls.popes[ind - 1]
        return None

    @classmethod
    def pope_match(cls, value, key=lambda pope: pope, cmp=lambda pope, value: pope.in_inauguration(value),
                   error="warn"):
        """
        Return all popet.Pope objects stored in self.popes which
        cmp(key(Pope), value) is True.
        if key is not provided, key is lambda pope: pope
        if cmp is not provided, cmp is lambda pope, value: pope.in_inauguration(value)

        error sets error level
            "ignore": ignore all errors occurred while running compare
            "warn": just warn error - default
            "raise": raise any errors

        Default, this will return Pope which in inauguration at given value(which must be datetime.date).
        """
        popes = []
        for pope in cls.popes:
            try:
                if cmp(key(pope), value):
                    popes.append(pope)
            except Exception:
                if error == "warn":
                    warn("There was error running cmp(key(Era), value) but skipped because ignore_error=True")
                elif error == "raise":
                    raise
        return popes
